from ctypes import c_void_p, c_char_p

from .lib import lib

auth_service_new = lib.auth_service_new
auth_service_new.restype = c_void_p

auth_service_delete = lib.auth_service_delete
auth_service_delete.argtypes = [c_void_p]

auth_service_create = lib.auth_service_create
auth_service_create.argtypes = [c_char_p, c_char_p, c_char_p]
auth_service_create.restype = c_void_p

auth_service_print = lib.auth_service_print
auth_service_print.argtypes = [c_void_p]



from ctypes import c_void_p, c_char_p
from ctypes import c_int, c_uint, c_int64, c_bool

from .lib import lib

SILVERSE_AUTH_TYPE_NONE = 0
SILVERSE_AUTH_TYPE_TOKEN = 1
SILVERSE_AUTH_TYPE_ACTION = 2
SILVERSE_AUTH_TYPE_ROLE = 3
SILVERSE_AUTH_TYPE_SERVICE = 4
SILVERSE_AUTH_TYPE_PERMISSIONS = 5
SILVERSE_AUTH_TYPE_MULTIPLE = 6
SILVERSE_AUTH_TYPE_COMPLETE = 7

SILVERSE_AUTH_SCOPE_NONE = 0
SILVERSE_AUTH_SCOPE_SINGLE = 1
SILVERSE_AUTH_SCOPE_MANAGEMENT = 2

silverse_auth_type_to_string = lib.silverse_auth_type_to_string
silverse_auth_type_to_string.argtypes = [c_int]
silverse_auth_type_to_string.restype = c_char_p

silverse_auth_scope_to_string = lib.silverse_auth_scope_to_string
silverse_auth_scope_to_string.argtypes = [c_int]
silverse_auth_scope_to_string.restype = c_char_p

silverse_auth_delete = lib.silverse_auth_delete
silverse_auth_delete.argtypes = [c_void_p]

silverse_auth_get_type = lib.silverse_auth_get_type
silverse_auth_get_type.argtypes = [c_void_p]
silverse_auth_get_type.restype = c_int

silverse_auth_get_scope = lib.silverse_auth_get_scope
silverse_auth_get_scope.argtypes = [c_void_p]
silverse_auth_get_scope.restype = c_int

silverse_auth_get_admin = lib.silverse_auth_get_admin
silverse_auth_get_admin.argtypes = [c_void_p]
silverse_auth_get_admin.restype = c_bool

silverse_auth_get_token_id = lib.silverse_auth_get_token_id
silverse_auth_get_token_id.argtypes = [c_void_p]
silverse_auth_get_token_id.restype = c_char_p

silverse_auth_get_token_type = lib.silverse_auth_get_token_type
silverse_auth_get_token_type.argtypes = [c_void_p]
silverse_auth_get_token_type.restype = c_int

silverse_auth_get_token_organization = lib.silverse_auth_get_token_organization
silverse_auth_get_token_organization.argtypes = [c_void_p]
silverse_auth_get_token_organization.restype = c_char_p

silverse_auth_get_token_permissions = lib.silverse_auth_get_token_permissions
silverse_auth_get_token_permissions.argtypes = [c_void_p]
silverse_auth_get_token_permissions.restype = c_char_p

silverse_auth_get_token_role = lib.silverse_auth_get_token_role
silverse_auth_get_token_role.argtypes = [c_void_p]
silverse_auth_get_token_role.restype = c_char_p

silverse_auth_get_token_user = lib.silverse_auth_get_token_user
silverse_auth_get_token_user.argtypes = [c_void_p]
silverse_auth_get_token_user.restype = c_char_p

silverse_auth_get_token_username = lib.silverse_auth_get_token_username
silverse_auth_get_token_username.argtypes = [c_void_p]
silverse_auth_get_token_username.restype = c_char_p

silverse_auth_get_mask = lib.silverse_auth_get_mask
silverse_auth_get_mask.argtypes = [c_void_p]
silverse_auth_get_mask.restype = c_int64

silverse_auth_create = lib.silverse_auth_create
silverse_auth_create.argtypes = [c_int]
silverse_auth_create.restype = c_void_p

silverse_auth_print_token = lib.silverse_auth_print_token
silverse_auth_print_token.argtypes = [c_void_p]

silverse_custom_authentication_handler = lib.silverse_custom_authentication_handler
silverse_custom_authentication_handler.argtypes = [c_void_p, c_void_p]
silverse_custom_authentication_handler.restype = c_uint



from cerver.utils.log import LOG_TYPE_NONE, cerver_log_both

from .lib import lib

SILVERSE_VERSION = "0.1.1"
SILVERSE_VERSION_NAME = "Version 0.1.1"
SILVERSE_VERSION_DATE = "31/03/2023"
SILVERSE_VERSION_TIME = "10:54 CST"
SILVERSE_VERSION_AUTHOR = "Erick Salas"

version = {
	"id": SILVERSE_VERSION,
	"name": SILVERSE_VERSION_NAME,
	"date": SILVERSE_VERSION_DATE,
	"time": SILVERSE_VERSION_TIME,
	"author": SILVERSE_VERSION_AUTHOR
}

silverse_libauth_version_print_full = lib.silverse_libauth_version_print_full
silverse_libauth_version_print_version_id = lib.silverse_libauth_version_print_version_id
silverse_libauth_version_print_version_name = lib.silverse_libauth_version_print_version_name

def pysilverse_version_print_full ():
	output = "\nPySilverse Version: {name}\n" \
		"Release Date: {date} - {time}\n" \
		"Author: {author}\n".format (**version)

	cerver_log_both (
		LOG_TYPE_NONE, LOG_TYPE_NONE,
		output.encode ("utf-8")
	)

def pysilverse_version_print_version_id ():
	cerver_log_both (
		LOG_TYPE_NONE, LOG_TYPE_NONE,
		f"\nPySilverse Version ID: {version.id}\n".encode ("utf-8")
	)

def pysilverse_version_print_version_name ():
	cerver_log_both (
		LOG_TYPE_NONE, LOG_TYPE_NONE,
		f"\nPySilverse Version: {version.name}\n".encode ("utf-8")
	)



from ctypes import c_void_p, c_char_p, c_int

from cerver.http import HTTP_ROUTE_AUTH_TYPE_CUSTOM
from cerver.http import RequestMethod
from cerver.http import HttpHandler
from cerver.http import http_route_create
from cerver.http import http_route_set_custom_data
from cerver.http import http_route_set_delete_custom_data
from cerver.http import http_route_set_auth
from cerver.http import http_route_set_authentication_handler

from cerver.http import http_cerver_enable_admin_routes
from cerver.http import http_cerver_admin_routes_set_custom_data
from cerver.http import http_cerver_admin_routes_set_delete_custom_data
from cerver.http import http_cerver_enable_admin_routes_authentication
from cerver.http import http_cerver_admin_routes_set_authentication_handler

from .lib import lib

from .auth import SILVERSE_AUTH_SCOPE_SINGLE
from .auth import SILVERSE_AUTH_SCOPE_MANAGEMENT
from .auth import silverse_custom_authentication_handler

auth_route_delete = lib.auth_route_delete
auth_route_delete.argtypes = [c_void_p]

auth_route_create = lib.auth_route_create
auth_route_create.restype = c_void_p

auth_route_create_action = lib.auth_route_create_action
auth_route_create_action.argtypes = [c_char_p]
auth_route_create_action.restype = c_void_p

auth_route_create_role = lib.auth_route_create_role
auth_route_create_role.argtypes = [c_char_p, c_char_p]
auth_route_create_role.restype = c_void_p

auth_route_print = lib.auth_route_print
auth_route_print.argtypes = [c_void_p]

# create route with common values
def silverse_route_create_internal (
	method: RequestMethod, path: str, handler: HttpHandler
) -> c_void_p:
	silverse_route = http_route_create (method, path.encode ("utf-8"), handler)

	http_route_set_auth (silverse_route, HTTP_ROUTE_AUTH_TYPE_CUSTOM)
	# http_route_set_delete_custom_data (silverse_route, auth_route_delete)
	http_route_set_authentication_handler (silverse_route, silverse_custom_authentication_handler)

	return silverse_route

def silverse_route_create (
	method: RequestMethod, path: str, handler: HttpHandler
) -> c_void_p:
	silverse_route = silverse_route_create_internal (method, path, handler)

	http_route_set_custom_data (silverse_route, auth_route_create ())

	return silverse_route

def silverse_action_route_create (
	method: RequestMethod, path: str, handler: HttpHandler, action: str
) -> c_void_p:
	action_route = silverse_route_create_internal (method, path, handler)

	http_route_set_custom_data (
		action_route, auth_route_create_action (action.encode ("utf-8"))
	)

	return action_route

def silverse_role_route_create (
	method: RequestMethod, path: str, handler: HttpHandler, role: str, action: str
) -> c_void_p:
	role_route = silverse_route_create_internal (method, path, handler)

	http_route_set_custom_data (
		role_route, auth_route_create_role (
			action.encode ("utf-8"), role.encode ("utf-8")
		)
	)

	return role_route

def silverse_admin_routes_configuration (http_cerver: c_void_p):
	http_cerver_enable_admin_routes (http_cerver, True)
	http_cerver_enable_admin_routes_authentication (http_cerver, HTTP_ROUTE_AUTH_TYPE_CUSTOM)
	http_cerver_admin_routes_set_custom_data (http_cerver, auth_route_create ())
	# http_cerver_admin_routes_set_delete_custom_data (http_cerver, auth_route_delete)
	http_cerver_admin_routes_set_authentication_handler (
		http_cerver, silverse_custom_authentication_handler
	)
